@@ -1,42 +1,33 @@
 # -*- encoding: utf-8 -*-
-import atexit
 import logging
 import os
-import sys
-from typing import AnyStr
-from typing import Dict
 from typing import List
 from typing import Optional
-import warnings
+
+import attr
 
 import ddtrace
 from ddtrace.internal import agent
+from ddtrace.internal import atexit
 from ddtrace.internal import service
 from ddtrace.internal import uwsgi
+from ddtrace.internal import writer
+from ddtrace.internal.utils import formats
 from ddtrace.profiling import collector
 from ddtrace.profiling import exporter
 from ddtrace.profiling import recorder
 from ddtrace.profiling import scheduler
 from ddtrace.profiling.collector import memalloc
 from ddtrace.profiling.collector import stack
+from ddtrace.profiling.collector import stack_event
 from ddtrace.profiling.collector import threading
 from ddtrace.profiling.exporter import file
 from ddtrace.profiling.exporter import http
-from ddtrace.utils import deprecation
-from ddtrace.vendor import attr
 
 
 LOG = logging.getLogger(__name__)
 
 
-def _get_api_key():
-    legacy = os.environ.get("DD_PROFILING_API_KEY")
-    if legacy:
-        deprecation.deprecation("DD_PROFILING_API_KEY", "Use DD_API_KEY")
-        return legacy
-    return os.environ.get("DD_API_KEY")
-
-
 def _get_service_name():
     for service_name_var in ("DD_SERVICE", "DD_SERVICE_NAME", "DATADOG_SERVICE_NAME"):
         service_name = os.environ.get(service_name_var)
@@ -44,15 +35,6 @@ def _get_service_name():
             return service_name
 
 
-def gevent_patch_all(event):
-    if "ddtrace.profiling.auto" in sys.modules:
-        warnings.warn(
-            "Starting the profiler before using gevent monkey patching is not supported "
-            "and is likely to break the application. Use DD_GEVENT_PATCH_ALL=true to avoid this.",
-            RuntimeWarning,
-        )
-
-
 class Profiler(object):
     """Run profiling while code is executed.
 
@@ -73,7 +55,7 @@ class Profiler(object):
 
         if profile_children:
             try:
-                uwsgi.check_uwsgi(self.start, atexit=self.stop if stop_on_exit else None)
+                uwsgi.check_uwsgi(self._restart_on_fork, atexit=self.stop if stop_on_exit else None)
             except uwsgi.uWSGIMasterProcess:
                 # Do nothing, the start() method will be called in each worker subprocess
                 return
@@ -97,12 +79,21 @@ class Profiler(object):
 
         :param flush: Flush last profile.
         """
-        self._profiler.stop(flush)
+        atexit.unregister(self.stop)
+        try:
+            self._profiler.stop(flush)
+        except service.ServiceStatusError:
+            # Not a best practice, but for backward API compatibility that allowed to call `stop` multiple times.
+            pass
 
     def _restart_on_fork(self):
         # Be sure to stop the parent first, since it might have to e.g. unpatch functions
         # Do not flush data as we don't want to have multiple copies of the parent profile exported.
-        self.stop(flush=False)
+        try:
+            self._profiler.stop(flush=False)
+        except service.ServiceStatusError:
+            # This can happen in uWSGI mode: the children won't have the _profiler started from the master process
+            pass
         self._profiler = self._profiler.copy()
         self._profiler.start()
 
@@ -135,43 +126,6 @@ class Profiler(object):
         return self._profiler.tags
 
 
-ENDPOINT_TEMPLATE = "https://intake.profile.{}"
-
-
-def _get_default_url(
-    tracer,  # type: Optional[ddtrace.Tracer]
-    api_key,  # type: str
-):
-    # type: (...) -> str
-    """Get default profiler exporter URL.
-
-    If an API key is not specified, the URL will default to the agent location configured via the environment variables.
-
-    If an API key is specified, the profiler goes into agentless mode and uses `DD_SITE` to upload directly to Datadog
-    backend.
-
-    :param api_key: The API key provided by the user.
-    :param tracer: The tracer object to use to find default URL.
-    """
-    # Default URL changes if an API_KEY is provided in the env
-    if api_key is None:
-        if tracer is None:
-            return agent.get_trace_url()
-        else:
-            # Only use the tracer writer URL if it has been modified by the user.
-            if tracer.writer.agent_url != agent.get_trace_url() and tracer.writer.agent_url != agent.DEFAULT_TRACE_URL:
-                return tracer.writer.agent_url
-            else:
-                return agent.get_trace_url()
-    # Agentless mode
-    legacy = os.environ.get("DD_PROFILING_API_URL")
-    if legacy:
-        deprecation.deprecation("DD_PROFILING_API_URL", "Use DD_SITE")
-        return legacy
-    site = os.environ.get("DD_SITE", "datadoghq.com")
-    return ENDPOINT_TEMPLATE.format(site)
-
-
 @attr.s
 class _ProfilerInstance(service.Service):
     """A instance of the profiler.
@@ -187,44 +141,52 @@ class _ProfilerInstance(service.Service):
     env = attr.ib(factory=lambda: os.environ.get("DD_ENV"))
     version = attr.ib(factory=lambda: os.environ.get("DD_VERSION"))
     tracer = attr.ib(default=ddtrace.tracer)
+    api_key = attr.ib(factory=lambda: os.environ.get("DD_API_KEY"), type=Optional[str])
+    agentless = attr.ib(factory=lambda: formats.asbool(os.environ.get("DD_PROFILING_AGENTLESS", "False")), type=bool)
 
     _recorder = attr.ib(init=False, default=None)
     _collectors = attr.ib(init=False, default=None)
     _scheduler = attr.ib(init=False, default=None)
 
-    @staticmethod
-    def _build_default_exporters(
-        tracer,  # type: Optional[ddtrace.Tracer]
-        url,  # type: Optional[str]
-        tags,  # type: Dict[str, AnyStr]
-        service,  # type: Optional[str]
-        env,  # type: Optional[str]
-        version,  # type: Optional[str]
-    ):
+    ENDPOINT_TEMPLATE = "https://intake.profile.{}"
+
+    def _build_default_exporters(self):
         # type: (...) -> List[exporter.Exporter]
         _OUTPUT_PPROF = os.environ.get("DD_PROFILING_OUTPUT_PPROF")
         if _OUTPUT_PPROF:
             return [
-                file.PprofFileExporter(_OUTPUT_PPROF),  # type: ignore[call-arg]
+                file.PprofFileExporter(_OUTPUT_PPROF),
             ]
 
-        api_key = _get_api_key()
-
-        if api_key is None:
-            # Agent mode
-            endpoint_path = "/profiling/v1/input"
+        if self.url is not None:
+            endpoint = self.url
+        elif self.agentless:
+            LOG.warning(
+                "Agentless uploading is currently for internal usage only and not officially supported. "
+                "You should not enable it unless somebody at Datadog instructed you to do so."
+            )
+            endpoint = self.ENDPOINT_TEMPLATE.format(os.environ.get("DD_SITE", "datadoghq.com"))
         else:
-            endpoint_path = "/v1/input"
+            if isinstance(self.tracer.writer, writer.AgentWriter):
+                endpoint = self.tracer.writer.agent_url
+            else:
+                endpoint = agent.get_trace_url()
 
-        endpoint = _get_default_url(tracer, api_key) if url is None else url
+        if self.agentless:
+            endpoint_path = "/v1/input"
+        else:
+            # Agent mode
+            # path is relative because it is appended
+            # to the agent base path.
+            endpoint_path = "profiling/v1/input"
 
         return [
-            http.PprofHTTPExporter(  # type: ignore[call-arg]
-                service=service,
-                env=env,
-                tags=tags,
-                version=version,
-                api_key=api_key,
+            http.PprofHTTPExporter(
+                service=self.service,
+                env=self.env,
+                tags=self.tags,
+                version=self.version,
+                api_key=self.api_key,
                 endpoint=endpoint,
                 endpoint_path=endpoint_path,
             ),
@@ -234,8 +196,8 @@ class _ProfilerInstance(service.Service):
         r = self._recorder = recorder.Recorder(
             max_events={
                 # Allow to store up to 10 threads for 60 seconds at 100 Hz
-                stack.StackSampleEvent: 10 * 60 * 100,
-                stack.StackExceptionSampleEvent: 10 * 60 * 100,
+                stack_event.StackSampleEvent: 10 * 60 * 100,
+                stack_event.StackExceptionSampleEvent: 10 * 60 * 100,
                 # (default buffer size / interval) * export interval
                 memalloc.MemoryAllocSampleEvent: int(
                     (memalloc.MemoryCollector._DEFAULT_MAX_EVENTS / memalloc.MemoryCollector._DEFAULT_INTERVAL) * 60
@@ -252,9 +214,7 @@ class _ProfilerInstance(service.Service):
             threading.LockCollector(r, tracer=self.tracer),
         ]
 
-        exporters = self._build_default_exporters(
-            self.tracer, self.url, self.tags, self.service, self.env, self.version
-        )
+        exporters = self._build_default_exporters()
 
         if exporters:
             self._scheduler = scheduler.Scheduler(
@@ -276,8 +236,8 @@ class _ProfilerInstance(service.Service):
             service=self.service, env=self.env, version=self.version, tracer=self.tracer, tags=self.tags
         )
 
-    def _start(self):
-        # type: () -> None
+    def _start_service(self):  # type: ignore[override]
+        # type: (...) -> None
         """Start the profiler."""
         collectors = []
         for col in self._collectors:
@@ -294,15 +254,15 @@ class _ProfilerInstance(service.Service):
         if self._scheduler is not None:
             self._scheduler.start()
 
-    def stop(self, flush=True):
+    def _stop_service(  # type: ignore[override]
+        self, flush=True  # type: bool
+    ):
+        # type: (...) -> None
         """Stop the profiler.
 
         :param flush: Flush a last profile.
         """
-        if self.status != service.ServiceStatus.RUNNING:
-            return
-
-        if self._scheduler:
+        if self._scheduler is not None:
             self._scheduler.stop()
             # Wait for the export to be over: export might need collectors (e.g., for snapshot) so we can't stop
             # collectors before the possibly running flush is finished.
@@ -312,14 +272,11 @@ class _ProfilerInstance(service.Service):
                 self._scheduler.flush()
 
         for col in reversed(self._collectors):
-            col.stop()
+            try:
+                col.stop()
+            except service.ServiceStatusError:
+                # It's possible some collector failed to start, ignore failure to stop
+                pass
 
         for col in reversed(self._collectors):
             col.join()
-
-        # Python 2 does not have unregister
-        if hasattr(atexit, "unregister"):
-            # You can unregister a method that was not registered, so no need to do any other check
-            atexit.unregister(self.stop)
-
-        super(_ProfilerInstance, self).stop()
